/*	injection_isrs.h

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

/* Header file multiple inclusion protection courtesy eclipse Header Template	*/
/* and http://gcc.gnu.org/onlinedocs/gcc-3.1.1/cpp/ C pre processor manual		*/
#ifndef FILE_INJECTION_ISRS_H_SEEN
#define FILE_INJECTION_ISRS_H_SEEN

#include "freeems.h"
#include "interrupts.h"

/* Masks for clearing flags and checking state of pin and therefore which change of state just occured */
#define MAIN1 BIT2
#define MAIN2 BIT3
#define MAIN3 BIT4
#define MAIN4 BIT5
#define MAIN5 BIT6
#define MAIN6 BIT7

/* Shortcuts for the registers such that it's readable and can be copy/pasted easily with less chance for errors */
#define MAIN1REG TCTL2
#define MAIN2REG TCTL2
#define MAIN3REG TCTL1
#define MAIN4REG TCTL1
#define MAIN5REG TCTL1
#define MAIN6REG TCTL1

/* Masks to be used with |= to switch a channel to "turn on on compare" mode */
#define MAIN1GOHIGH BIT4
#define MAIN2GOHIGH BIT6
#define MAIN3GOHIGH BIT0
#define MAIN4GOHIGH BIT2
#define MAIN5GOHIGH BIT4
#define MAIN6GOHIGH BIT6

/* Masks to be used with &= to switch a channel to "turn off on compare" mode */
#define MAIN1GOLOW ~MAIN1GOHIGH
#define MAIN2GOLOW ~MAIN2GOHIGH
#define MAIN3GOLOW ~MAIN3GOHIGH
#define MAIN4GOLOW ~MAIN4GOHIGH
#define MAIN5GOLOW ~MAIN5GOHIGH
#define MAIN6GOLOW ~MAIN6GOHIGH

/* Use port K for staged injector output */
#define STAGEDPORT PORTK

/* Masks for setting and checking stagedOn status var and turning the channel on */
#define STAGED1ON BIT0 
#define STAGED2ON BIT1 
#define STAGED3ON BIT2 
#define STAGED4ON BIT3 
#define STAGED5ON BIT4 
#define STAGED6ON BIT5

/* Masks for unsetting stagedOn status var and turning the channel off */
#define STAGED1OFF ~STAGED1ON 
#define STAGED2OFF ~STAGED2ON 
#define STAGED3OFF ~STAGED3ON 
#define STAGED4OFF ~STAGED4ON 
#define STAGED5OFF ~STAGED5ON 
#define STAGED6OFF ~STAGED6ON 

/* Internal use to decide if staged is actually required or not based on pulsewidth etc */
unsigned char stagedRequired;

#else
	/* let us know if we are being untidy with headers */
	#warning "Header file INJECTION_ISRS_H seen before, sort it out!"
/* end of the wrapper ifdef from the very top */
#endif
