/*	main.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "main.h"

void inject(int channel)
{
	// Toggle the correct cylinders channel on or off.
	if(channel == 0){
		Injector1();
	}else if(channel == 1){
		Injector2();
	}else if(channel == 2){
		Injector3();
	}else if(channel == 3){
		Injector4();
	}else if(channel == 4){
		Injector5();
	}else if(channel == 5){
		Injector6();
	}
}

void Injector1(void)
{
	if(!localFuelCut){
		PTT ^= 0x04;
		if(localStagedInjection){
			PORTK ^= 0x01;
		}
	}
	if(!localSparkCut){
		if(phase){
			PORTB ^= 0x40;
		}else{
			PORTB ^= 0x01;
		}
	}
}

void Injector2(void)
{
	if(!localFuelCut){
		PTT ^= 0x08;
		if(localStagedInjection){
			PORTK ^= 0x02;
		}
	}
	if(!localSparkCut){
		if(phase){
			PORTB ^= 0x80;
		}else{
			PORTB ^= 0x02;
		}
	}
}

void Injector3(void)
{
	if(!localFuelCut){
		PTT ^= 0x10;
		if(localStagedInjection){
			PORTK ^= 0x04;
		}
	}
	if(!localSparkCut){
		if(phase){
			PORTA ^= 0x01;
		}else{
			PORTB ^= 0x04;
		}
	}
}

void Injector4(void)
{
	if(!localFuelCut){
		PTT ^= 0x20;
		if(localStagedInjection){
			PORTK ^= 0x08;
		}
	}
	if (!localSparkCut) {
		if(phase){
			PORTA ^= 0x02;
		}else{
			PORTB ^= 0x08;
		}
	}
}

void Injector5(void)
{
	if(!localFuelCut){
		PTT ^= 0x40;
		if(localStagedInjection){
			PORTK ^= 0x10;
		}
	}
	if(!localSparkCut){
		if(phase){
			PORTA ^= 0x04;
		}else{
			PORTB ^= 0x10;
		}
	}
}

void Injector6(void)
{
	if(!localFuelCut){
		PTT ^= 0x80;
		if(localStagedInjection){
			PORTK ^= 0x20;
		}
	}
	if(!localSparkCut){
		if(phase){
			PORTA ^= 0x08;
		}else{
			PORTB ^= 0x20;
		}
	}
}

int  main(){
	// Set everything up.
	init();
	
	// intitial PWM settings
	PWMPER0 = 0xFF; // 255 for ADC0 testing
	PWMPER1 = 0xFF; // 255 for ADC1 testing
	PWMPER2 = 15;
	PWMPER3 = 31;
	PWMPER4 = 63;
	PWMPER5 = 127;
	
	PWMDTY5 = 1;
	
	// Run forever repeating.
	while(TRUE){
		/* Flash the rest of port A (3 are initialised off and one on, so they alternate) */
		PORTA ^= 0xF0;
		
		/* Clear the ign lines so that when sparkCut is enabled we can use it as an output variable display register set */
		if(sparkCut == 0){
			PORTB = ZEROS;
		}

		// Take a copy of the vars before each loop such that the LEDs stay in consistent state during changes.
		unsigned char localEvents = combustionEventsPerEngineCycle;
		unsigned char localPorts = ports;

		// Flash the injector channel leds on and off repeatedly in a strobed sequence.
		int i;
		for(i=0;i < localEvents;i++)
		{
			int channel;
			
			// update the toggle variables before each inner loop iteration
			localStagedInjection = 0; //stagedInjection;
			localSparkCut = sparkCut;
			localFuelCut = 1; //fuelCut;

			// on once per cycle for J0 (tacho)
			PORTJ ^= 0x01;
			
			// set the phase and channel based on cylinder count and port count
			if(localEvents <= 6 || (i + localPorts) < localEvents){
				phase = 0;
				channel = i;
			}else{
				phase = 1;
				channel = i - localPorts;
			}

			// Turn channel on.
			inject(channel);

			// Snooze for dual adc dependent time.
			sleep2();
			
			// and off again too (tacho)
			PORTJ ^= 0x01;
			
			// PWM experimentation
			//PWMPER0 += 1;
			//PWMPER1 += 1;
			PWMPER2 += 1;
			PWMPER3 += 1;
			PWMPER4 += 1;
			PWMPER5 += 1;
			
			PWMDTY0 = ATD0DR0 >> 2; // scale raw adc to a duty
			PWMDTY1 = ATD0DR1 >> 2; // scale raw adc to a duty
			PWMDTY2 = PWMPER2 / 8;
			PWMDTY3 = (3 * PWMPER3) / 4;
			PWMDTY4 = (7 * PWMPER4) / 8;
			PWMDTY5 -= 1;
			
			// Snooze that time again.
			sleep2();
			
			// Turn channel off.
			inject(channel);
			
			// Next cylinder...			
		}
	}
}
