/*	misc_isrs.c

	Copyright 2008 Fred Cooke

	This file is part of the FreeEMS project.

	FreeEMS software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	FreeEMS software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any FreeEMS software.  If not, see <http://www.gnu.org/licenses/>.

	We ask that if you make any changes to this file you send them upstream to us at admin@diyefi.org

	Thank you for choosing FreeEMS to run your engine! */

#include "misc_isrs.h"

extern void sleep(int ms);

void UISR(void){}				/* Unimplemented interrupt service routine */

void PortPISR(void){}			/* Port P interrupt service routine */
void PortJISR(void){}			/* Port J interrupt service routine */

/* Port H interrupt service routine */
void PortHISR(void)
{
	// read the interrupt flags to a variable
	unsigned char portHFlags = PIFH;
	PORTM = PIFH; // display the raw flag set
	portHFlags &= 0xF0; // mask out the other bits
	
	// debounce
	if(portHDebounce == 0){
		portHDebounce = 2;
	}else{
		// clear all flags (too soon)
		PIFH = ONES;
		return;
	}

	// let the world know that we are doing anything at all by switching the sequence of port A flashing
	PORTA ^= 0x80;
	
	// find out which pin triggered it, clear the flag, perform the action.
	switch(portHFlags)
	{
	case 0x80 :	// Increment cylinder count and set port count appropriately.
		switch (combustionEventsPerEngineCycle) {
			case 1 :
				combustionEventsPerEngineCycle = 2;
				ports = 2;
				break;
			case 2 :
				combustionEventsPerEngineCycle = 3;
				ports = 3;
				break;
			case 3 :
				combustionEventsPerEngineCycle = 4;
				ports = 4;
				break;
			case 4 :
				combustionEventsPerEngineCycle = 5;
				ports = 5;
				break;
			case 5 :
				combustionEventsPerEngineCycle = 6;
				ports = 6;
				break;
			case 6 :
				combustionEventsPerEngineCycle = 8;
				ports = 4;
				break;
			case 8 :
				combustionEventsPerEngineCycle = 10;
				ports = 5;
				break;
			case 10 :
				combustionEventsPerEngineCycle = 12;
				ports = 6;
				break;
			case 12 :
				combustionEventsPerEngineCycle = 1;
				ports = 1;
				break;
		}
		break;
	case 0x40 : // Injection output enable/disable
		if(fuelCut == 0){
			fuelCut = 1;
		}else{
			fuelCut = 0;
		}
		break;
	case 0x20 : // Ignition output enable/disable
		if(sparkCut == 0){
			sparkCut = 1;
		}else{
			sparkCut = 0;
		}
		break;
	case 0x10 : // Staged injection enable/disable
		if(stagedInjection == 0){
			stagedInjection = 1;
		}else{
			stagedInjection = 0;
		}
		break;
	default : // Nothing to do except wait for another button press
		break;
	}	

	// clear all flags (we only want one at a time)
	PIFH = ONES;
}

void IRQISR(void){}				/* IRQ/PE1 interrupt service routine */
void XIRQISR(void){}			/* XIRQ/PE0 interrupt service routine */

void PIT2ISR(void){}			/* PIT timer 2 interrupt service routine */
void PIT3ISR(void){}			/* PIT timer 3 interrupt service routine */

void TimerOverflow(void){
	/* Clear the timer overflow interrupt flag */
	TFLG2 = 0x80;
	
	/* Increment the timer extension variable */
	timerExtension += 1;
	
	// cycle a LED so we know the frequency it is at...
	PORTE ^= 0x80; // PK7 on and off
}
